@@ -1,27 +1,52 @@
 # checked in 2013-06-05 by J. Fox
 # 2014-09-04: J. Fox: empty groups produce warning rather than error
+# 2016-10-16: J. Fox: add option for adaptive kernel.
+# 2016-11-26: J. Fox: rejig for pure-R adaptive kernel
 
 densityPlot <- function(x, ...){
     UseMethod("densityPlot")
 }
 
-densityPlot.default <- function (x, g, bw="SJ", adjust=1,
-    kernel = c("gaussian", "epanechnikov", "rectangular", 
-               "triangular", "biweight", "cosine", "optcosine"),
+densityPlot.default <- function (x, g, method=c("kernel", "adaptive"), 
+    bw=if (method == "adaptive") bw.nrd0 else "SJ", adjust=1,
+    kernel,
+    xlim,
+    ylim,
+    normalize=FALSE,
     xlab=deparse(substitute(x)), ylab="Density", 
     col=palette(), lty=seq_along(col), lwd=2, grid=TRUE,
     legend.location="topright", legend.title=deparse(substitute(g)), show.bw=FALSE,
     rug=TRUE, ...) {
-    ylab
+    norm <- function(x, y){
+        n <- length(x)
+        x0 <- diff(range(x))/(n - 1)
+        y0 <- (y[1:(n-1)] + y[2:n])/2
+        exp(log(y) - log(sum(y0*x0)))
+    }
+    method <- match.arg(method)
+    if (method == "kernel"){
+        kernel <- if (missing(kernel)) "gaussian"
+        else match.arg(kernel, c("gaussian", "epanechnikov", "rectangular", 
+                                      "triangular", "biweight", "cosine", "optcosine"))
+    }
+    else{
+        if(missing(kernel)) kernel <- dnorm
+        if (!is.function(kernel)) stop("for the adaptive kernel estimator, the kernel argument must be a function")
+    }
+    force(ylab)
+    force(xlab)
     if (!is.numeric(x)) stop("argument x must be numeric")
-    kernel <- match.arg(kernel)
     if (missing(g)) {
-        density <- density(x, bw=bw, adjust=adjust, kernel=kernel)
+        density <- if (method == "adaptive") adaptiveKernel(x, bw=bw, adjust=adjust, ...)
+            else density(x, bw=bw, adjust=adjust, kernel=kernel, ...)
+        if (normalize) density$y <- norm(density$x, density$y)
+        if (missing(xlim)) xlim <- range(density$x)
+        if (missing(ylim)) ylim <- c(0, max(density$y))
         if (show.bw) xlab <- paste(xlab, " (bandwidth = ", format(density$bw), ")", sep="")
-        plot(density, xlab=xlab, ylab=ylab, main="", type="n", ...)
+        plot(xlim, ylim, xlab=xlab, ylab=ylab, main="", type="n")
         if (rug) rug(x)
         if (grid) grid()
-        lines(density, lwd=lwd)
+        lines(density, col=col[1], lwd=lwd, lty=lty[1], xlim=xlim, ylim=ylim)
     }
     else {
         if (!is.factor(g)) stop("argument g must be a factor")
@@ -36,19 +61,26 @@ densityPlot.default <- function (x, g, bw="SJ", adjust=1,
         x <- x[valid]
         g <- g[valid]
         levels <- levels(g)
-        if (length(bw) == 1) bw <- rep(bw, length(levels))
+        if (is.numeric(bw) && length(bw) == 1) bw <- rep(bw, length(levels))
         if (length(adjust) == 1) adjust <- rep(adjust, length(levels))
-        if (length(bw) != length(levels)) stop("number of entries in bw be 1 or must equal number of groups")
+        if (is.numeric(bw) && length(bw) != length(levels)) stop("number of entries in bw be 1 or must equal number of groups")
         if (length(adjust) != length(levels)) stop("number of entries in adjust must be 1 or must equal number of groups")
         densities <- vector(length(levels), mode="list") 
-        names(bw) <- names(adjust) <- names(densities) <- levels
+        names(adjust) <- names(densities) <- levels
+        if (is.numeric(bw)) names(bw) <- levels
         for (group in levels){
-            densities[[group]] <- density(x[g == group], bw=bw[group], 
-                                          adjust=adjust[group], kernel=kernel)
+            densities[[group]] <- if (method == "adaptive") 
+                adaptiveKernel(x[g == group], bw=if (is.numeric(bw)) bw[group] else bw, adjust=adjust[group], ...)
+                else density(x[g == group], bw=if (is.numeric(bw)) bw[group] else bw, adjust=adjust[group], kernel=kernel, ...)
+            if (normalize) densities[[group]]$y <- norm(densities[[group]]$x, densities[[group]]$y)
         }
-        range.x <- range(unlist(lapply(densities, function(den) range(den$x))))
-        max.y <- max(sapply(densities, function(den) max(den$y)))
-        plot(range.x, c(0, max.y), xlab=xlab, ylab=ylab, type="n", ...)
+        if (missing(xlim)){
+            xlim <- range(unlist(lapply(densities, function(den) range(den$x))))
+        }
+        if (missing(ylim)){
+            ylim <- c(0, max(sapply(densities, function(den) max(den$y))))
+        }
+        plot(xlim, ylim, xlab=xlab, ylab=ylab, type="n")
         if (grid) grid()
         for (i in 1:length(levels)){
             lines(densities[[i]]$x, densities[[i]]$y, lty=lty[i], col=col[i], lwd=lwd)
@@ -65,7 +97,7 @@ densityPlot.default <- function (x, g, bw="SJ", adjust=1,
             for (i in 1:length(levels)) rug(x[g == levels[i]], col=col[i])
         }
     }
-    return(invisible(NULL))
+    return(invisible(if (missing(g)) density else densities))
 }
 
 densityPlot.formula <- function(formula, data=NULL, subset, na.action=NULL, xlab, ylab, ...){
